@@ -83,9 +83,11 @@ class Feature:
 
     Args:
         feature_names: features are stored as columns in a data frame,
-            ``feature_names`` defines the names of the columns.
-            If ``num_channels`` > 1,
-            the channel number will be appended to the column names.
+            where ``feature_names`` defines the names of the columns.
+            If ``len(channels)`` > 1,
+            the data frame has a multi-column index with
+            with channel ID as first level
+            and ``feature_names`` as second level
         name: name of the feature set, e.g. ``'stft'``
         params: parameters that describe the feature set,
             e.g. ``{'win_size': 512, 'hop_size': 256, 'num_fft': 512}``.
@@ -176,7 +178,7 @@ class Feature:
             and ``sampling_rate is None``
         ValueError: if ``hop_dur`` is specified, but not ``win_dur``
 
-    Example:
+    Examples:
         >>> def mean_std(signal, sampling_rate):
         ...     return [signal.mean(), signal.std()]
         >>> interface = Feature(['mean', 'std'], process_func=mean_std)
@@ -188,6 +190,7 @@ class Feature:
                                 mean       std
         start  end
         0 days 0 days 00:00:01   2.0  0.816497
+        >>> # Apply interface on an audformat conform index of a dataframe
         >>> import audb
         >>> db = audb.load(
         ...     'emodb',
@@ -201,20 +204,48 @@ class Feature:
                                                            mean       std
         file            start  end
         wav/03a01Fa.wav 0 days 0 days 00:00:01.898250 -0.000311  0.082317
-        >>> interface_with_sliding_window = Feature(
+        >>> # Apply interface with a sliding window
+        >>> interface = Feature(
         ...     ['mean', 'std'],
         ...     process_func=mean_std,
         ...     process_func_applies_sliding_window=False,
         ...     win_dur=1.0,
         ...     hop_dur=0.25,
         ... )
-        >>> interface_with_sliding_window.process_index(index, root=db.root)
+        >>> interface.process_index(index, root=db.root)
                                                                            mean       std
         file            start                  end
         wav/03a01Fa.wav 0 days 00:00:00        0 days 00:00:01        -0.000329  0.098115
                         0 days 00:00:00.250000 0 days 00:00:01.250000 -0.000405  0.087917
                         0 days 00:00:00.500000 0 days 00:00:01.500000 -0.000285  0.067042
                         0 days 00:00:00.750000 0 days 00:00:01.750000 -0.000187  0.063677
+        >>> # Apply the same process function on all channels
+        >>> # of a multi-channel signal
+        >>> import audiofile
+        >>> signal, sampling_rate = audiofile.read(
+        ...     audeer.path(db.root, db.files[0]),
+        ...     always_2d=True,
+        ... )
+        >>> signal_multi_channel = np.concatenate(
+        ...     [
+        ...         signal - 0.5,
+        ...         signal + 0.5,
+        ...     ],
+        ... )
+        >>> interface = Feature(
+        ...     ['mean', 'std'],
+        ...     process_func=mean_std,
+        ...     process_func_is_mono=True,
+        ...     channels=[0, 1],
+        ... )
+        >>> interface.process_signal(
+        ...     signal_multi_channel,
+        ...     sampling_rate,
+        ... )
+                                              0                   1
+                                           mean       std      mean       std
+        start  end
+        0 days 0 days 00:00:01.898250 -0.500311  0.082317  0.499689  0.082317
 
     """  # noqa: E501
     @deprecated_process_func_applies_sliding_window_default_value()
@@ -278,12 +309,13 @@ class Feature:
         feature_names = audeer.to_list(feature_names)
         if num_channels > 1:
             column_names = []
-            for channel in range(num_channels):
+            for channel in channels:
                 column_names.extend(
-                    [f'{name}-{channel}' for name in feature_names]
+                    [(channel, feature_name) for feature_name in feature_names]
                 )
+            column_names = pd.MultiIndex.from_tuples(column_names)
         else:
-            column_names = feature_names
+            column_names = pd.Index(feature_names)
 
         if process_func is None:
             def process_func(signal, _):
@@ -501,6 +533,8 @@ class Feature:
         When called again with the same index,
         features will be read from the cached file.
 
+        .. _audformat: https://audeering.github.io/audformat/data-format.html
+
         Args:
             index: index with segment information
             root: root folder to expand relative file paths
@@ -513,8 +547,6 @@ class Feature:
                 but ``win_dur`` is not set
             ValueError: if index is not conform to audformat_
 
-        .. _audformat: https://audeering.github.io/audformat/data-format.html
-
         """
         cache_path = None
 
@@ -620,8 +652,11 @@ class Feature:
             self,
             frame: pd.DataFrame,
     ) -> np.ndarray:
-        r"""Return feature values as a :class:`numpy.ndarray` in original
-        shape, i.e. ``(channels, features, time)``.
+        r"""Return feature values as a numpy array.
+
+        The returned :class:`numpy.ndarray`
+        has the original shape,
+        i.e. ``(channels, features, time)``.
 
         Args:
             frame: feature frame
