@@ -91,7 +91,7 @@ class Process:
         ValueError: if ``resample = True``, but ``sampling_rate = None``
         ValueError: if ``hop_dur`` is specified, but not ``win_dur``
 
-    Example:
+    Examples:
         >>> def mean(signal, sampling_rate):
         ...     return signal.mean()
         >>> interface = Process(process_func=mean)
@@ -102,6 +102,7 @@ class Process:
         start   end
         0 days  0 days 00:00:01   2.0
         dtype: float64
+        >>> # Apply interface on an audformat conform index of a dataframe
         >>> import audb
         >>> db = audb.load(
         ...     'emodb',
@@ -115,6 +116,7 @@ class Process:
         file             start   end
         wav/03a01Fa.wav  0 days  0 days 00:00:01.898250    -0.000311
         dtype: float32
+        >>> # Apply interface with a sliding window
         >>> interface = Process(
         ...     process_func=mean,
         ...     win_dur=1.0,
