@@ -1,5 +1,9 @@
-import subprocess
 from datetime import date
+import os
+import shutil
+import subprocess
+
+import audeer
 
 
 # Project -----------------------------------------------------------------
@@ -20,12 +24,20 @@ title = f'{project} Documentation'
 master_doc = 'index'
 extensions = []
 source_suffix = '.rst'
-exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store', '**.ipynb_checkpoints']
+exclude_patterns = [
+    'build',
+    'tests',
+    'Thumbs.db',
+    '.DS_Store',
+    'api-src',
+]
+templates_path = ['_templates']
 pygments_style = None
 extensions = [
     'jupyter_sphinx',
     'sphinx.ext.autodoc',
     'sphinx.ext.napoleon',  # support for Google-style docstrings
+    'sphinx.ext.autosummary',
     'sphinx_autodoc_typehints',
     'sphinx.ext.viewcode',
     'sphinx.ext.intersphinx',
@@ -49,6 +61,8 @@ linkcheck_ignore = [
 autodoc_mock_imports = [
     'tqdm',
 ]
+autodoc_inherit_docstrings = True
+
 # Reference with :ref:`data-header:Database`
 autosectionlabel_prefix_document = True
 autosectionlabel_maxdepth = 2
@@ -57,6 +71,9 @@ autosectionlabel_maxdepth = 2
 copybutton_prompt_text = r'>>> |\.\.\. |$ '
 copybutton_prompt_is_regexp = True
 
+# Disable auto-generation of TOC entries in the API
+# https://github.com/sphinx-doc/sphinx/issues/6316
+toc_object_entries = False
 
 # HTML --------------------------------------------------------------------
 html_theme = 'sphinx_audeering_theme'
@@ -69,3 +86,13 @@ html_context = {
     'display_github': True,
 }
 html_title = title
+
+# Copy API (sub-)module RST files to docs/api/ folder ---------------------
+audeer.mkdir('api')
+api_src_files = audeer.list_file_names('api-src')
+api_dst_files = [
+    audeer.path('api', os.path.basename(src_file))
+    for src_file in api_src_files
+]
+for src_file, dst_file in zip(api_src_files, api_dst_files):
+    shutil.copyfile(src_file, dst_file)
