@@ -8,6 +8,7 @@ import pytest
 
 import audinterface
 import audformat
+import audobject
 
 
 def signal_duration(signal, sampling_rate):
@@ -18,18 +19,17 @@ def signal_max(signal, sampling_rate):
     return np.max(signal)
 
 
+class SignalObject(audobject.Object):
+
+    def __call__(self, signal, sampling_rate):
+        return np.max(signal)
+
+
 SEGMENT = audinterface.Segment(
     process_func=lambda x, sr:
-        pd.MultiIndex.from_arrays(
-            [
-                [
-                    pd.to_timedelta(0),
-                ],
-                [
-                    pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
-                ],
-            ],
-            names=['start', 'end'],
+        audinterface.utils.signal_index(
+            pd.to_timedelta(0),
+            pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
         )
 )
 
@@ -330,7 +330,7 @@ def test_process_file(
     root = str(tmpdir.mkdir('wav'))
     file = 'file.wav'
     path = os.path.join(root, file)
-    af.write(path, signal, sampling_rate)
+    af.write(path, signal, sampling_rate, bit_depth=32)
 
     # test absolute path
     y = process.process_file(
@@ -503,19 +503,48 @@ def test_process_folder(
         verbose=False,
     )
     sampling_rate = 8000
-    path = str(tmpdir.mkdir('wav'))
+    root = str(tmpdir.mkdir('wav'))
     files = [
-        os.path.join(path, f'file{n}.wav') for n in range(num_files)
+        os.path.join(root, f'file{n}.wav') for n in range(num_files)
     ]
     for file in files:
         signal = np.random.uniform(-1.0, 1.0, (1, sampling_rate))
         af.write(file, signal, sampling_rate)
-    y = process.process_folder(path)
+    y = process.process_folder(root)
     pd.testing.assert_series_equal(
         y,
         process.process_files(files),
     )
 
+    # non-existing folder
+    with pytest.raises(FileNotFoundError):
+        process.process_folder('bad-folder')
+
+    # empty folder
+    root = str(tmpdir.mkdir('empty'))
+    y = process.process_folder(root)
+    pd.testing.assert_series_equal(y, pd.Series(dtype=object))
+
+
+def test_process_func_args():
+    def process_func(s, sr, arg1, arg2):
+        assert arg1 == 'foo'
+        assert arg2 == 'bar'
+    audinterface.Process(
+        process_func=process_func,
+        process_func_args={
+            'arg1': 'foo',
+            'arg2': 'bar',
+        }
+    )
+    with pytest.warns(UserWarning):
+        audinterface.Process(
+            feature_names=('o1', 'o2', 'o3'),
+            process_func=process_func,
+            arg1='foo',
+            arg2='bar',
+        )
+
 
 @pytest.mark.parametrize(
     'num_workers, multiprocessing',
@@ -527,6 +556,8 @@ def test_process_folder(
 )
 def test_process_index(tmpdir, num_workers, multiprocessing):
 
+    cache_root = os.path.join(tmpdir, 'cache')
+
     process = audinterface.Process(
         process_func=None,
         sampling_rate=None,
@@ -593,9 +624,32 @@ def test_process_index(tmpdir, num_workers, multiprocessing):
         )
         np.testing.assert_equal(signal, value)
 
+    # cache result
+    y = process.process_index(
+        index,
+        root=root,
+        cache_root=cache_root,
+    )
+    os.remove(path)
+
+    # fails because second file does not exist
+    with pytest.raises(RuntimeError):
+        process.process_index(
+            index,
+            root=root,
+        )
+
+    # loading from cache still works
+    y_cached = process.process_index(
+        index,
+        root=root,
+        cache_root=cache_root,
+    )
+    pd.testing.assert_series_equal(y, y_cached)
+
 
 @pytest.mark.parametrize(
-    'process_func, process_func_kwargs, segment, signal, '
+    'process_func, process_func_args, segment, signal, '
     'sampling_rate, file, start, end, keep_nat, expected_signal',
     [
         (
@@ -818,7 +872,7 @@ def test_process_index(tmpdir, num_workers, multiprocessing):
 )
 def test_process_signal(
         process_func,
-        process_func_kwargs,
+        process_func_args,
         segment,
         signal,
         sampling_rate,
@@ -835,7 +889,7 @@ def test_process_signal(
         segment=segment,
         keep_nat=keep_nat,
         verbose=False,
-        **process_func_kwargs,
+        process_func_args=process_func_args,
     )
     x = process.process_signal(
         signal,
@@ -874,9 +928,7 @@ def test_process_signal(
     if file is None:
         y = pd.Series(
             [expected_signal],
-            index=pd.MultiIndex.from_arrays(
-                [[start], [end]], names=['start', 'end']
-            ),
+            index=audinterface.utils.signal_index(start, end),
         )
     else:
         y = pd.Series(
@@ -901,49 +953,37 @@ def test_process_signal(
             None,
             np.random.random(5 * 44100),
             44100,
-            pd.MultiIndex.from_arrays(
-                [
-                    pd.to_timedelta([]),
-                    pd.to_timedelta([]),
-                ],
-                names=['start', 'end']
-            ),
+            audinterface.utils.signal_index(),
         ),
         (
             None,
             np.random.random(5 * 44100),
             44100,
-            pd.MultiIndex.from_arrays(
-                [
-                    pd.timedelta_range('0s', '3s', 3),
-                    pd.timedelta_range('1s', '4s', 3),
-                ],
-                names=['start', 'end']
+            audinterface.utils.signal_index(
+                pd.timedelta_range('0s', '3s', 3),
+                pd.timedelta_range('1s', '4s', 3)
             ),
         ),
         (
             signal_max,
             np.random.random(5 * 44100),
             44100,
-            pd.MultiIndex.from_arrays(
-                [
-                    pd.timedelta_range('0s', '3s', 3),
-                    pd.timedelta_range('1s', '4s', 3),
-                ],
-                names=['start', 'end']
+            audinterface.utils.signal_index(
+                pd.timedelta_range('0s', '3s', 3),
+                pd.timedelta_range('1s', '4s', 3),
             ),
         ),
         (
             signal_max,
             np.random.random(5 * 44100),
             44100,
-            pd.MultiIndex.from_arrays(
-                [
-                    pd.to_timedelta([]),
-                    pd.to_timedelta([]),
-                ],
-                names=['start', 'end']
-            ),
+            audinterface.utils.signal_index(),
+        ),
+        (
+            SignalObject(),
+            np.random.random(5 * 44100),
+            44100,
+            audinterface.utils.signal_index(),
         ),
         pytest.param(
             signal_max,
@@ -1007,7 +1047,7 @@ def test_process_signal_from_index(
     if not expected:
         pd.testing.assert_series_equal(
             result,
-            pd.Series([], index, dtype=float),
+            pd.Series([], index, dtype=object),
         )
     else:
         pd.testing.assert_series_equal(
@@ -1016,58 +1056,406 @@ def test_process_signal_from_index(
         )
 
 
+@pytest.mark.parametrize(
+    'process_func, signal, sampling_rate, min_signal_dur, '
+    'max_signal_dur, expected',
+    [
+        (
+            None,
+            np.ones((1, 44100)),
+            44100,
+            None,
+            None,
+            np.ones((1, 44100)),
+        ),
+        (
+            None,
+            np.ones((1, 44100)),
+            44100,
+            2,
+            None,
+            np.concatenate(
+                [
+                    np.ones((1, 44100)),
+                    np.zeros((1, 44100)),
+                ],
+                axis=1,
+            ),
+        ),
+        (
+            None,
+            np.ones((1, 44100)),
+            44100,
+            None,
+            0.01,
+            np.ones((1, 441)),
+        ),
+    ]
+)
+def test_process_signal_min_max(
+        process_func,
+        signal,
+        sampling_rate,
+        min_signal_dur,
+        max_signal_dur,
+        expected,
+):
+    process = audinterface.Process(
+        process_func=process_func,
+        sampling_rate=None,
+        resample=False,
+        min_signal_dur=min_signal_dur,
+        max_signal_dur=max_signal_dur,
+        verbose=False,
+    )
+    result = process.process_signal(signal, sampling_rate)
+    expected = pd.Series(
+        [expected],
+        index=audinterface.utils.signal_index(
+            pd.to_timedelta(0),
+            pd.to_timedelta(expected.shape[1] / sampling_rate, unit='s'),
+        )
+    )
+    pd.testing.assert_series_equal(result, expected)
+
+
+@pytest.mark.parametrize(
+    'process_func, signal, sampling_rate',
+    [
+        (
+            lambda x, sr: x.mean(),
+            np.array([0, 0, 0, 0, 1, 1, 1, 1], dtype=np.float32),
+            1,
+        ),
+    ]
+)
+@pytest.mark.parametrize(
+    'start, end, win_dur, hop_dur, expected',
+    [
+        (
+            None, None, 4, None,
+            pd.Series(
+                [0, 0.5, 1],
+                audinterface.utils.signal_index(
+                    [0, 2, 4],
+                    [4, 6, 8],
+                ),
+                dtype=np.float32,
+            ),
+        ),
+        (
+            None, None, 4, 2,
+            pd.Series(
+                [0, 0.5, 1],
+                audinterface.utils.signal_index(
+                    [0, 2, 4],
+                    [4, 6, 8],
+                ),
+                dtype=np.float32,
+            ),
+        ),
+        (
+            None, None, 4, 3,
+            pd.Series(
+                [0, 0.75],
+                audinterface.utils.signal_index(
+                    [0, 3],
+                    [4, 7],
+                ),
+                dtype=np.float32,
+            ),
+        ),
+        (
+            None, None, 4, 4,
+            pd.Series(
+                [0, 1],
+                audinterface.utils.signal_index(
+                    [0, 4],
+                    [4, 8],
+                ),
+                dtype=np.float32,
+            ),
+        ),
+        (
+            None, None, 2, 4,
+            pd.Series(
+                [0, 1.0],
+                audinterface.utils.signal_index(
+                    [0, 4],
+                    [2, 6],
+                ),
+                dtype=np.float32,
+            ),
+        ),
+        (
+            1.0, None, 4, 2,
+            pd.Series(
+                [0.25, 0.75],
+                audinterface.utils.signal_index(
+                    [1, 3],
+                    [5, 7],
+                ),
+                dtype=np.float32,
+            ),
+        ),
+        (
+            1.0, 5.0, 4, 2,
+            pd.Series(
+                [0.25],
+                audinterface.utils.signal_index(1, 5),
+                dtype=np.float32,
+            ),
+        ),
+        (
+            1.0, 2.0, 4, 2,
+            pd.Series(
+                [],
+                audinterface.utils.signal_index(),
+                dtype=object,
+            ),
+        ),
+        (
+            9.0, 15.0, 4, 2,
+            pd.Series(
+                [],
+                audinterface.utils.signal_index(),
+                dtype=object,
+            ),
+        ),
+        # missing win duration
+        pytest.param(
+            None, None, None, 2, None,
+            marks=pytest.mark.xfail(raises=ValueError),
+        )
+    ]
+)
+def test_process_with_sliding_window(
+        tmpdir,
+        process_func,
+        signal,
+        sampling_rate,
+        start,
+        end,
+        win_dur,
+        hop_dur,
+        expected,
+):
+    # save signal to file
+    root = tmpdir
+    file = 'file.wav'
+    path = os.path.join(root, file)
+    audiofile.write(path, signal, sampling_rate, bit_depth=32)
+
+    # create interface
+    process = audinterface.Process(
+        process_func=process_func,
+        hop_dur=hop_dur,
+        win_dur=win_dur,
+    )
+
+    # process signal
+    y = process.process_signal(
+        signal,
+        sampling_rate,
+        start=start,
+        end=end,
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # process signal from index
+    y = process.process_signal_from_index(
+        signal,
+        sampling_rate,
+        expected.index,
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # add file to expected index
+    expected.index = audformat.segmented_index(
+        [file] * len(expected.index),
+        expected.index.get_level_values('start'),
+        expected.index.get_level_values('end'),
+    )
+
+    # process signal with file
+    y = process.process_signal(
+        signal,
+        sampling_rate,
+        file=file,
+        start=start,
+        end=end,
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # process file
+    y = process.process_file(file, start=start, end=end, root=root)
+    pd.testing.assert_series_equal(y, expected)
+
+    # process index
+    y = process.process_index(expected.index, root=root)
+    pd.testing.assert_series_equal(y, expected)
+
+
+def test_process_with_special_args(tmpdir):
+
+    duration = 3
+    sampling_rate = 1
+    signal = np.zeros((2, duration), np.float32)
+    num_files = 10
+    win_dur = 1
+    num_frames = duration // win_dur
+    num_workers = 3
+
+    # create files
+    root = tmpdir
+    files = [f'f{idx}.wav' for idx in range(num_files)]
+    index = audformat.segmented_index(
+        np.repeat(files, num_frames),
+        np.tile(range(num_frames), num_files),
+        np.tile(range(1, num_frames + 1), num_files),
+    )
+    for file in files:
+        path = os.path.join(root, file)
+        audiofile.write(path, signal, sampling_rate, bit_depth=32)
+
+    # create interface
+    def process_func(signal, sampling_rate, idx, file, root):
+        return (idx, file, root)
+
+    process = audinterface.Process(
+        process_func=process_func,
+        num_workers=num_workers,
+    )
+
+    # process signal
+    y = process.process_signal(signal, sampling_rate)
+    expected = pd.Series(
+        [(0, None, None)],
+        audinterface.utils.signal_index(0, duration),
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # process signal from index
+    y = process.process_signal_from_index(
+        signal,
+        sampling_rate,
+        expected.index,
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # process file
+    y = process.process_file(files[0], root=root)
+    expected = pd.Series(
+        [(0, files[0], root)],
+        audformat.segmented_index(files[0], 0, duration),
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # process files
+    y = process.process_files(files, root=root)
+    expected = pd.Series(
+        [(idx, files[idx], root) for idx in range(num_files)],
+        audformat.segmented_index(
+            files,
+            [0] * num_files,
+            [duration] * num_files,
+        ),
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # process index with a filewise index
+    y = process.process_index(
+        audformat.filewise_index(files),
+        root=root,
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # process index with a segmented index
+    y = process.process_index(index, root=root)
+    expected = pd.Series(
+        [(idx, file, root) for idx, (file, _, _) in enumerate(index)],
+        index,
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # sliding window
+    # frames belonging to the same files have same idx
+    process = audinterface.Process(
+        process_func=process_func,
+        win_dur=win_dur,
+        hop_dur=win_dur,
+        num_workers=num_workers,
+    )
+    y = process.process_files(files, root=root)
+    values = []
+    for idx in range(num_files):
+        file = files[idx]
+        for _ in range(num_frames):
+            values.append((idx, file, root))
+    expected = pd.Series(values, index)
+    pd.testing.assert_series_equal(y, expected)
+
+    # mono processing function
+    # returns
+    # [((0, files[0], root), (0, files[0], root)),
+    #  ((1, files[1], root), (1, files[1], root)),
+    #  ... ]
+    process = audinterface.Process(
+        process_func=process_func,
+        process_func_is_mono=True,
+        num_workers=num_workers,
+    )
+    y = process.process_index(index, root=root)
+    expected = pd.Series(
+        [((idx, file, root), (idx, file, root))
+         for idx, (file, _, _) in enumerate(index)],
+        index,
+    )
+    pd.testing.assert_series_equal(y, expected)
+
+    # explicitely pass special arguments
+
+    process = audinterface.Process(
+        process_func=process_func,
+        process_func_args={'idx': 99, 'file': 'my/file', 'root': None},
+        num_workers=num_workers,
+    )
+    y = process.process_index(index, root=root)
+    expected = pd.Series([(99, 'my/file', None)] * len(index), index)
+    pd.testing.assert_series_equal(y, expected)
+
+
 @pytest.mark.parametrize(
     'segment',
     [
+        audinterface.Segment(
+            process_func=lambda x, sr: audinterface.utils.signal_index()
+        ),
         audinterface.Segment(
             process_func=lambda x, sr:
-                pd.MultiIndex.from_arrays(
-                    [pd.to_timedelta([]), pd.to_timedelta([])],
-                    names=['start', 'end'],
+                audinterface.utils.signal_index(
+                    pd.to_timedelta(0),
+                    pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
                 )
         ),
         audinterface.Segment(
             process_func=lambda x, sr:
-                pd.MultiIndex.from_arrays(
-                    [
-                        [
-                            pd.to_timedelta(0),
-                        ],
-                        [
-                            pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
-                        ],
-                    ],
-                    names=['start', 'end'],
-                )
-        ),
-        audinterface.Segment(
-            process_func=lambda x, sr:
-            pd.MultiIndex.from_arrays(
-                [
-                    [
-                        pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
-                    ],
-                    [
-                        pd.to_timedelta(x.shape[1] / sr, unit='s'),
-                    ],
-                ],
-                names=['start', 'end'],
+            audinterface.utils.signal_index(
+                pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
+                pd.to_timedelta(x.shape[1] / sr, unit='s'),
             )
         ),
         audinterface.Segment(
             process_func=lambda x, sr:
-                pd.MultiIndex.from_arrays(
+                audinterface.utils.signal_index(
                     [
-                        [
-                            pd.to_timedelta(0),
-                            pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
-                        ],
-                        [
-                            pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
-                            pd.to_timedelta(x.shape[1] / sr),
-                        ],
+                        pd.to_timedelta(0),
+                        pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
+                    ],
+                    [
+                        pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
+                        pd.to_timedelta(x.shape[1] / sr),
                     ],
-                    names=['start', 'end'],
                 )
         )
     ]
@@ -1192,5 +1580,5 @@ def test_sampling_rate_mismatch(
         resample=resample,
         verbose=False,
     )
-    signal = np.array([1., 2., 3.])
+    signal = np.array([1., 2., 3.]).astype('float32')
     process.process_signal(signal, signal_sampling_rate)
